@@ -2,7 +2,10 @@ from datetime  import datetime
 from functools import partial as functools_partial
 
 import netCDF4
-
+if netCDF4.__version__ >= '1.2.5':
+    from netCDF4 import netcdftime
+    _netCDF4_netcdftime_parse_date = netcdftime._parse_date
+    
 from numpy import around    as numpy_around
 from numpy import array     as numpy_array
 from numpy import ndarray   as numpy_ndarray
@@ -15,10 +18,10 @@ from .units     import Units
 if netCDF4.__version__ <= '1.1.1':
     _netCDF4_netcdftime_parse_date = netCDF4.netcdftime._parse_date
 #    _netCDF4_netcdftime_strftime   = netCDF4.netcdftime._strftime
-else:
+elif netCDF4.__version__ <= '1.2.4':
     _netCDF4_netcdftime_parse_date = netCDF4.netcdftime.netcdftime._parse_date
 #    _netCDF4_netcdftime_strftime   = netCDF4.netcdftime._datetime._strftime
-
+    
 
 # Define some useful units
 _calendar_years  = Units('calendar_years')
@@ -151,7 +154,8 @@ True
                                            dayofyr)
             self.microsecond = 0
 
-        self.calendar = calendar
+            #        if netCDF4.__version__ <= '1.2.4':
+        self._calendar = calendar
     #--- End: def
 
     def __deepcopy__(self, memo):
@@ -172,7 +176,7 @@ x.__hash__() <==> hash(x)
         '''
         return hash((self.__class__, self.year, self.month, self.day, self.hour,
                      self.minute, self.second, self.microsecond,
-                     self.calendar))
+                     self._calendar))
     #--- End: def
 
     def __repr__(self):
@@ -182,7 +186,7 @@ x__repr__() <==> repr(x)
 
 '''   
         out = '<CF {0}: {1}>'.format(self.__class__.__name__, self)
-        calendar = self.calendar
+        calendar = self._calendar
         if calendar is not None:
             out = out.replace('>', ' '+calendar+'>')
         return out
@@ -308,7 +312,7 @@ Return a deep copy.
 
 '''
         return type(self)(*self.timetuple()[:6], microsecond=self.microsecond,
-                          calendar=self.calendar)
+                          calendar=self._calendar)
     #--- End: def
 
     def elements(self):
@@ -338,7 +342,7 @@ d.minute, d.second, d.microsecond)``.
         except:
             return False
             
-        return self.calendar == getattr(other, 'calendar', None)
+        return self._calendar == getattr(other, '_calendar', None)
     #--- End: def
 
     def replace(self, year=None, month=None, day=None, hour=None,
@@ -351,7 +355,7 @@ d.minute, d.second, d.microsecond)``.
               for i, j in zip((year, month, day, hour,
                                minute, second, microsecond),
                               self.elements())],
-            calendar=(calendar if calendar is not None else self.calendar)
+            calendar=(calendar if calendar is not None else self._calendar)
         )
     #--- End: def
 
@@ -1044,7 +1048,7 @@ minute, second and microsecond.
         # cf.dt('2001-3-4')
         # cf.dt('2001-3-4', calendar='noleap')
         out = st2Datetime(arg0)
-        out.calendar = calendar
+        out._calendar = calendar
         return out
         
     if isinstance(arg0, Datetime):
@@ -1053,8 +1057,8 @@ minute, second and microsecond.
         # cf.dt(cf.Datetime(2001, 3, 4), calendar='noleap')
         # cf.dt(cf.Datetime(2001, 3, 4, calendar='360_day'), calendar='noleap')
         out = arg0.copy()
-        if out.calendar is None or calendar is not None:
-            out.calendar = calendar
+        if out._calendar is None or calendar is not None:
+            out._calendar = calendar
         return out
 
     if hasattr(arg0, 'timetuple'):
