@@ -39,7 +39,7 @@ class NetCDFError(Exception):
     pass
 
 def write(fields, filename, fmt='NETCDF3_CLASSIC', overwrite=True,
-          verbose=False, cfa_options=None, mode='w',
+          verbose=True, cfa_options=None, mode='w',
           least_significant_digit=None, endian='native', compress=0,
           fletcher32=False, no_shuffle=False, datatype=None,
           single=False, double=False, reference_datetime=None,
@@ -340,6 +340,10 @@ and auxiliary coordinate roles for different data variables.
          # 
          # ------------------------------------------------------------
          'unlimited': unlimited,
+         # -----------------------------------------------------------
+         # Miscellaneous
+         # -----------------------------------------------------------
+         'verbose': verbose,
     }
 
     if fmt == 'CFA3':
@@ -426,9 +430,9 @@ and auxiliary coordinate roles for different data variables.
         # Reset HDF chunking
         f.HDF_chunks(org_chunks)
 
-        if verbose:
-            for e in f:            
-                print repr(e)
+#        if verbose == 1:
+#            for e in f:            
+#                print repr(e)
     #-- End: for
 
     # ---------------------------------------------------------------
@@ -1707,6 +1711,9 @@ created. The ``seen`` dictionary is updated for *cfvar*.
     None
 
 '''
+    if g['verbose']:
+        print repr(cfvar)+' netCDF: '+ncvar
+
     # Set the netCDF4.createVariable datatype
     datatype = _datatype(cfvar, g=g)
     ncdimensions = dimensions
@@ -1865,6 +1872,9 @@ message+". Unlimited dimension must be the first (leftmost) dimension of the var
 
             convert_dtype = g['datatype']
 
+            if data.fits_in_one_chunk_in_memory(data.dtype.itemsize):
+                data.varray
+
             for partition in data.partitions.flat:
                 array = partition.dataarray(**pda_args)   
 
