@@ -1386,7 +1386,7 @@ Create ISO 8601 time interval strings:
         '''   
         def _dHMS(duration, dt, end):
             units = Units('{0} since {1}'.format(duration.Units.units, dt),
-                          dt.calendar)
+                          dt._calendar)
             dt1 = Data(0.0, units) 
 
             if not end:
