@@ -34,7 +34,7 @@ class DataTest(unittest.TestCase):
 #    test_only = ('test_Data_any',)
 #    test_only = ('test_Data_mean',)
 #    test_only = ('test_Data_sums_of_weights',)
-#    test_only = ('test_Data_sample_size','test_Data_sd_var')
+    test_only = ['test_Data_sd_var']
 #    test_only = ('test_Data_sd_var',)
 #    test_only = ['test_Data_max_min_sum']
 #    test_only = ('test_Data___setitem__',)
@@ -55,12 +55,13 @@ class DataTest(unittest.TestCase):
     ones = numpy.ones(a.shape, dtype=float)
     
     ma = numpy.ma.arange(-100, 200., dtype=float).reshape(3, 4, 5, 5)
-    ma[0, :, 2, 3] = numpy.ma.masked
-    ma[1, 2, 3, :] = numpy.ma.masked
-    ma[0, 3, :, 3] = numpy.ma.masked
     ma[:, 1, 4, 4] = numpy.ma.masked
+    ma[0, :, 2, 3] = numpy.ma.masked
+    ma[0, 3, :, 3] = numpy.ma.masked
+    ma[1, 2, 3, :] = numpy.ma.masked
 
-    mw = numpy.ma.array(w)
+#    mw = numpy.ma.array(w)
+    mw = numpy.ma.array(w, mask=ma.mask)
 
     mones = numpy.ma.array(ones, mask=ma.mask)
    
@@ -1197,13 +1198,70 @@ class DataTest(unittest.TestCase):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
 
-        ddofs=(0, 1, 2)
-        f = 2
+        ddofs=(0, 1) #(0, 1, 2)
+#        f = 2
 
         for chunksize in self.chunk_sizes:   
             for pp in (False, True):
                 cf.CHUNKSIZE(chunksize) 
 
+#                # unweighted, unmasked
+#                d = cf.Data(self.a, units='K')
+#                for np, h in zip((numpy.var, numpy.std),
+#                                 ('var'    , 'sd')):
+#                    for ddof in ddofs:
+#                        for axes in self.axes_combinations:
+#                            b = reshape_array(self.a, axes)
+#                            b = np(b, axis=-1, ddof=ddof)                
+#                            e = getattr(d, h)(axes=axes, squeeze=True,
+#                                              ddof=ddof, _preserve_partitions=pp)
+#                            self.assertTrue(
+#                                e.allclose(b, rtol=1e-05, atol=1e-08),
+#                                "%s, axis=%s, unweighted, unmasked \ne=%s, \nb=%s, \ne-b=%s" %
+#                                (h, axes, e.array, b, e.array-b))
+#                #--- End: for
+#    
+#                # unweighted, masked
+#                d = cf.Data(self.a, units='K')
+#                x = cf.Data(self.w)
+#                for h in ('var', 'sd'):
+#                    for axes in self.axes_combinations:                
+#                        for ddof in ddofs:
+#                            e = getattr(d, h)(axes=axes, squeeze=True, ddof=ddof)
+#        
+#                            b = reshape_array(self.a, axes)
+#        
+#                            not_enough_data = numpy.ma.count(b, axis=-1) <= ddof
+#        
+#                            avg = numpy.ma.average(b, axis=-1)
+#        
+#                            if numpy.ndim(avg) < b.ndim:
+#                                avg = numpy.ma.expand_dims(avg, -1)
+#        
+#                            b, sv = numpy.ma.average((b-avg)**2, axis=-1, returned=True)
+#        
+#                            b = numpy.ma.where(not_enough_data, numpy.ma.masked, b)
+#        
+#                            b *= sv/(sv-ddof)
+#                            if h == 'sd':
+#                                b **= 0.5
+#                            b = numpy.ma.asanyarray(b)
+#        
+#                            e = getattr(d, h)(axes=axes, squeeze=True,
+#                                              ddof=ddof,
+#                                              _preserve_partitions=pp)
+#                            
+#                            self.assertTrue(
+#                                (e.mask.array == b.mask).all(),
+#                                "%s, axes=%s, ddof=%s, \ne.mask=%s, \nb.mask=%s, \ne.mask==b.mask=%s" %
+#                                (h, axes, ddof, e.mask.array, b.mask, e.mask.array==b.mask))
+#                            self.assertTrue(
+#                                e.allclose(b, rtol=1e-05, atol=1e-08),
+#                                "%s, axes=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" %
+#                                (h, axes, ddof, e.array, b, e.array-b))
+#                #--- End: for
+    
+
                 # unweighted, unmasked
                 d = cf.Data(self.a, units='K')
                 for np, h in zip((numpy.var, numpy.std),
@@ -1216,111 +1274,108 @@ class DataTest(unittest.TestCase):
                                               ddof=ddof, _preserve_partitions=pp)
                             self.assertTrue(
                                 e.allclose(b, rtol=1e-05, atol=1e-08),
-                                "%s, axis=%s, unweighted, unmasked \ne=%s, \nb=%s, \ne-b=%s" %
-                                (h, axes, e.array, b, e.array-b))
+                                "%s, axis=%s, unweighted, unmasked pp=%s, \ne=%s, \nb=%s, \ne-b=%s" %
+                                (h, axes, pp, e.array, b, e.array-b))
                 #--- End: for
     
                 # unweighted, masked
-                d = cf.Data(self.a, units='K')
-                x = cf.Data(self.w)
-                for h in ('var', 'sd'):
-                    for axes in self.axes_combinations:                
-                        for ddof in ddofs:
-                            e = getattr(d, h)(axes=axes, squeeze=True, ddof=ddof)
-        
-                            b = reshape_array(self.a, axes)
-        
-                            not_enough_data = numpy.ma.count(b, axis=-1) <= ddof
-        
-                            avg = numpy.ma.average(b, axis=-1)
-        
-                            if numpy.ndim(avg) < b.ndim:
-                                avg = numpy.ma.expand_dims(avg, -1)
-        
-                            b, sv = numpy.ma.average((b-avg)**2, axis=-1, returned=True)
-        
-                            b = numpy.ma.where(not_enough_data, numpy.ma.masked, b)
-        
-                            b *= sv/(sv-ddof)
-                            if h == 'sd':
-                                b **= 0.5
-                            b = numpy.ma.asanyarray(b)
-        
+                d = cf.Data(self.ma, units='K')
+                for np, h in zip((numpy.ma.var, numpy.ma.std),
+                                 ('var'    , 'sd')):
+                    for ddof in ddofs:
+                        for axes in self.axes_combinations:
+                            b = reshape_array(self.ma, axes)
+                            b = np(b, axis=-1, ddof=ddof)                
                             e = getattr(d, h)(axes=axes, squeeze=True,
-                                              ddof=ddof,
-                                              _preserve_partitions=pp)
-                            
-                            self.assertTrue(
-                                (e.mask.array == b.mask).all(),
-                                "%s, axes=%s, ddof=%s, \ne.mask=%s, \nb.mask=%s, \ne.mask==b.mask=%s" %
-                                (h, axes, ddof, e.mask.array, b.mask, e.mask.array==b.mask))
+                                              ddof=ddof, _preserve_partitions=pp)
                             self.assertTrue(
                                 e.allclose(b, rtol=1e-05, atol=1e-08),
-                                "%s, axes=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" %
-                                (h, axes, ddof, e.array, b, e.array-b))
+                                "%s, axis=%s, unweighted, masked, pp=%s, \ne=%s, \nb=%s, \ne-b=%s" %
+                                (h, axes, pp, e.array, b, e.array-b))
                 #--- End: for
     
                 # weighted, unmasked 
-                f= 2
                 d = cf.Data(self.a, units='K')
                 x = cf.Data(self.w)
                 for h in ('var', 'sd'):
                     for axes in self.axes_combinations:                
-                        for ddof in ddofs:
+                        for ddof in (0, 1):
                             b = reshape_array(self.a, axes)
                             v = reshape_array(self.w, axes)
                             
                             avg = numpy.average(b, axis=-1, weights=v)
                             if numpy.ndim(avg) < b.ndim:
                                 avg = numpy.expand_dims(avg, -1)
-            
-                            b, sv = numpy.average((b-avg)**2, axis=-1, weights=v,
+                                
+                            b, V1 = numpy.average((b-avg)**2, axis=-1, weights=v,
                                                   returned=True)
-                            b *= f*sv/(f*sv-ddof)
-                            
+                                
+                            if ddof == 1:
+                                # Calculate the weighted unbiased variance. The unbiased
+                                # variance weighted with _reliability_ weights is
+                                # [V1**2/(V1**2-V2)]*var.
+                                V2 = numpy.asanyarray((v * v).sum(axis=-1))
+                                b *= (V1*V1/(V1*V1 - V2))
+                            elif ddof == 0:
+                                pass
+
                             if h == 'sd':
                                 b **= 0.5
-        
+
+                            b = numpy.ma.asanyarray(b)
+
                             e = getattr(d, h)(axes=axes, weights=x,
                                               squeeze=True, ddof=ddof,
-                                              a=f,
                                               _preserve_partitions=pp)
-        
+                            
                             self.assertTrue(
                                 e.allclose(b, rtol=1e-05, atol=1e-08) ,
-                                "%s, axis=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" % \
-                                (h, axes, ddof, e.array, b, e.array-b))
+                                "%s, axis=%s, weighted, unmasked, pp=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" % \
+                                (h, axes, pp, ddof, e.array, b, e.array-b))
                 #--- End: for
     
                 # weighted, masked
-                f = 2
                 d = cf.Data(self.ma, units='K')
                 x = cf.Data(self.w)
                 for h in ('var', 'sd'):
-                    for axes in self.axes_combinations:                
-                        for ddof in ddofs:
+                    for axes in self.axes_combinations:
+                        for ddof in (0, 1):
+#                            print '\nweighted masked', chunksize, pp, 'axes='+str(axes), 'ddof='+str(ddof), h
                             b = reshape_array(self.ma, axes)
                             v = reshape_array(self.mw, axes)
-        
+
+#                            print ' v.sum(<) =', v.sum()
+#                            print ' x.sum() =', x.array.sum()
+                            
                             not_enough_data = numpy.ma.count(b, axis=-1) <= ddof
-        
+
                             avg = numpy.ma.average(b, axis=-1, weights=v)
                             if numpy.ndim(avg) < b.ndim:
-                                avg = numpy.ma.expand_dims(avg, -1)
-        
-                            b, sv = numpy.ma.average((b-avg)**2, axis=-1, weights=v,
+                                avg = numpy.expand_dims(avg, -1)
+                                
+                            b, V1 = numpy.ma.average((b-avg)**2, axis=-1, weights=v,
                                                      returned=True)
-        
+#                            print 'V1.sum() =', V1.sum()
+                            
                             b = numpy.ma.where(not_enough_data, numpy.ma.masked, b)
-        
-                            b *= f*sv/(f*sv-ddof)
+
+                            if ddof == 1:
+                                # Calculate the weighted unbiased variance. The unbiased
+                                # variance weighted with _reliability_ weights is
+                                # [V1**2/(V1**2-V2)]*var.
+                                V2 = numpy.asanyarray((v * v).sum(axis=-1))
+#                                print 'V2.sum() =', V2.sum()
+                                b *= (V1*V1/(V1*V1 - V2))                                
+                            elif ddof == 0:
+                                pass
+
                             if h == 'sd':
                                 b **= 0.5
-                            b = numpy.ma.asanyarray(b)
-        
+#                            else:
+#                                print 'b.sum()=', b.sum()                                
+#                            print 'e', repr(x)
                             e = getattr(d, h)(axes=axes, weights=x,
                                               squeeze=True, ddof=ddof,
-                                              a=f,
                                               _preserve_partitions=pp)
         
                             if h == 'sd':
@@ -1335,11 +1390,94 @@ class DataTest(unittest.TestCase):
     
                             self.assertTrue(
                                 e.allclose(b, rtol=1e-05, atol=1e-08),
-                                "%s, axis=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" %
-                                (h, axes, ddof, e.array, b, e.array-b))
+"%s, axis=%s, weighted, masked, pp=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" %
+                                (h, axes, pp, ddof, e.array, b, e.array-b))
                 #--- End: for
             #--- End: for
         #--- End: for
+
+
+
+#                # weighted, unmasked 
+#                f= 2
+#                d = cf.Data(self.a, units='K')
+#                x = cf.Data(self.w)
+#                for h in ('var', 'sd'):
+#                    for axes in self.axes_combinations:                
+#                        for ddof in ddofs:
+#                            b = reshape_array(self.a, axes)
+#                            v = reshape_array(self.w, axes)
+#                            
+#                            avg = numpy.average(b, axis=-1, weights=v)
+#                            if numpy.ndim(avg) < b.ndim:
+#                                avg = numpy.expand_dims(avg, -1)
+#            
+#                            b, sv = numpy.average((b-avg)**2, axis=-1, weights=v,
+#                                                  returned=True)
+#                            b *= f*sv/(f*sv-ddof)
+#                            
+#                            if h == 'sd':
+#                                b **= 0.5
+#        
+#                            e = getattr(d, h)(axes=axes, weights=x,
+#                                              squeeze=True, ddof=ddof,
+##                                              a=f,
+#                                              _preserve_partitions=pp)
+#        
+#                            self.assertTrue(
+#                                e.allclose(b, rtol=1e-05, atol=1e-08) ,
+#                                "%s, axis=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" % \
+#                                (h, axes, ddof, e.array, b, e.array-b))
+#                #--- End: for
+#    
+#                # weighted, masked
+#                f = 2
+#                d = cf.Data(self.ma, units='K')
+#                x = cf.Data(self.w)
+#                for h in ('var', 'sd'):
+#                    for axes in self.axes_combinations:                
+#                        for ddof in ddofs:
+#                            b = reshape_array(self.ma, axes)
+#                            v = reshape_array(self.mw, axes)
+#        
+#                            not_enough_data = numpy.ma.count(b, axis=-1) <= ddof
+#        
+#                            avg = numpy.ma.average(b, axis=-1, weights=v)
+#                            if numpy.ndim(avg) < b.ndim:
+#                                avg = numpy.ma.expand_dims(avg, -1)
+#        
+#                            b, sv = numpy.ma.average((b-avg)**2, axis=-1, weights=v,
+#                                                     returned=True)
+#        
+#                            b = numpy.ma.where(not_enough_data, numpy.ma.masked, b)
+#        
+#                            b *= f*sv/(f*sv-ddof)
+#                            if h == 'sd':
+#                                b **= 0.5
+#                            b = numpy.ma.asanyarray(b)
+#        
+#                            e = getattr(d, h)(axes=axes, weights=x,
+#                                              squeeze=True, ddof=ddof,
+##                                              a=f,
+#                                              _preserve_partitions=pp)
+#        
+#                            if h == 'sd':
+#                                self.assertTrue(e.Units == d.Units)
+#                            else:                            
+#                                self.assertTrue(e.Units == d.Units**2)
+#                       
+#                            self.assertTrue(
+#                                (e.mask.array == b.mask).all(),
+#                                "%s, axis=%s, \ne.mask=%s, \nb.mask=%s, \ne.mask==b.mask=%s" %
+#                                (h, axes, e.mask.array, b.mask, e.mask.array==b.mask))
+#    
+#                            self.assertTrue(
+#                                e.allclose(b, rtol=1e-05, atol=1e-08),
+#                                "%s, axis=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" %
+#                                (h, axes, ddof, e.array, b, e.array-b))
+#                #--- End: for
+#            #--- End: for
+#        #--- End: for
         cf.CHUNKSIZE(self.original_chunksize)
     #--- End: def
 
