@@ -18,7 +18,8 @@ class FieldTest(unittest.TestCase):
         # self.test_only = ('test_Field_transpose','test_Field_squeeze',)
         # self.test_only = ('test_Field_collapse',)
         # self.test_only = ('test_Field_match','test_Field_items',)
-        # self.test_only = ('test_Field_items',)
+        # self.test_only = ['test_Field_match']
+        # self.test_only = ['test_Field_items']
         # self.test_only = ('test_Field_axes','test_Field_data_axes',)
         # self.test_only = ('test_Field_where',)
         # self.test_only = ('test_Field_anchor',)
@@ -123,7 +124,9 @@ class FieldTest(unittest.TestCase):
 
         f = cf.read(self.filename)[0]
 
-        self.assertTrue(set(f.items()) == set(['aux0','aux1','aux2','aux3','aux4','dim0','dim1','dim2','msr0','ref0','ref1']))
+        self.assertTrue(set(f.items()) == set(['aux0','aux1','aux2','aux3','aux4','dim0','dim1','dim2','msr0','ref0','ref1']),
+                        '{} != {}'.format(sorted(f.items()),
+                                          sorted(['aux0','aux1','aux2','aux3','aux4','dim0','dim1','dim2','msr0','ref0','ref1'])))
         self.assertTrue(set(f.items(inverse=True)) == set([]))
 
         self.assertTrue(set(f.items(ndim=1)) == set(['aux0','aux1','aux4', 'dim0','dim1','dim2']))
@@ -330,22 +333,26 @@ class FieldTest(unittest.TestCase):
             g = f.collapse('T: mean', group=360)
 
             for group in (cf.M(12), 
+                          cf.M(12, month=6),
                           cf.M(12, month=12),
                           cf.M(12, day=16),
-                          cf.M(12, year=1999, month=11, day=27)):
+                          cf.M(12, month=11, day=27)):
                 g = f.collapse('T: mean', group=group)
                 bound = g.coord('T').bounds.dtarray[0, 1]
-                self.assertTrue(bound.month == group.month, "group=%s" % group)
-                self.assertTrue(bound.day   == group.day  , "group=%s" % group)
+                self.assertTrue(bound.month == group.offset.month,
+                                "bound={!r}, group={!r}".format(bound, group))
+                self.assertTrue(bound.day == group.offset.day,
+                                "bound={!r}, group={!r}".format(bound, group))
             #--- End: for  
 
             for group in (cf.D(30), 
                           cf.D(30, month=12),
                           cf.D(30, day=16),
-                          cf.D(30, year=1999, month=11, day=27)):
+                          cf.D(30, month=11, day=27)):
                 g = f.collapse('T: mean', group=group)
                 bound = g.coord('T').bounds.dtarray[0, 1]
-                self.assertTrue(bound.day == group.day, "group=%s" % group)
+                self.assertTrue(bound.hour == group.offset.hour,
+                                "bound={!r}, group={!r}".format(bound, group))
             #--- End: for  
 
         #--- End: for    
@@ -402,6 +409,6 @@ if __name__ == '__main__':
 #    print 'cf-python version:'     , cf.__version__
 #    print 'cf-python path:'        , os.path.abspath(cf.__file__)
     print 'Run date:', datetime.datetime.now()
-    cf.ENVIRONMENT()
+    cf.environment()
     print''
     unittest.main(verbosity=2)
